import MagicString from "magic-string";
import { parse } from "svelte-parse-markup";
import { findStaticImports } from "mlly";
import { genObjectFromValues } from "knitwork";
import { loadAliases } from "./utils/alias.mjs";
import { getCssModule, getCssModuleImports } from "./utils/css-module.mjs";
import { resolveOptions } from "./options.mjs";
export function cssModules(_options = {}) {
  const options = resolveOptions(_options);
  const cssModuleCache = /* @__PURE__ */ new Map();
  return {
    markup({ content, filename }) {
      const ast = parse(content);
      if (ast.css == null) {
        const s = new MagicString(content);
        s.append("\n<style>\n</style>\n");
        return {
          code: s.toString(),
          map: s.generateMap({
            source: filename,
            includeContent: true
          })
        };
      }
    },
    async script({ content, filename }) {
      const aliases = await loadAliases();
      const s = new MagicString(content);
      const imports = findStaticImports(content).filter(
        (i) => ["module.css", "module.scss"].some((ext) => i.specifier.endsWith(ext))
      );
      const cssModuleImports = await getCssModuleImports({
        imports,
        aliases,
        filename
      });
      const cssModules2 = [];
      for (const cmi of cssModuleImports) {
        const cssModule = await getCssModule(cmi, options);
        cssModules2.push(cssModule);
        const obj = genObjectFromValues(cssModule.exports);
        const gen = `const ${cmi.defaultImport} = ${obj};
`;
        s.overwrite(cmi.imp.start, cmi.imp.end, gen);
      }
      if (filename != null) {
        cssModuleCache.set(filename, cssModules2);
      }
      return {
        code: s.toString(),
        map: s.generateMap({
          source: filename,
          includeContent: true
        })
      };
    },
    style({ content, filename }) {
      const s = new MagicString(content);
      if (filename == null) {
        return { code: content };
      }
      const cssModules2 = cssModuleCache.get(filename) ?? [];
      for (const cssModule of cssModules2) {
        const code = `
${options.includeOriginalPath === false ? "" : `/*${cssModule.path}*/`}
${cssModule.css}
`;
        s.append(code);
      }
      return {
        code: s.toString(),
        map: s.generateMap({
          source: filename,
          includeContent: true
        })
      };
    }
  };
}
